@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2003, 2005, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -11,14 +11,17 @@
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <math.h>
+#include <string.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
 #ifdef HAVE_CAIRO
 #include <cairo.h>
+#include <cairo-xlib.h>
 #endif
 
 #include "gpeclockface.h"
@@ -27,16 +30,24 @@
 
 #ifdef BACKGROUND_IMAGE
 static GdkPixbuf *clock_background, *clock_background_24;
-static guint background_width, background_height;
-#endif
-
 static GdkPixbuf *day_night_wheel;
+#endif
 
 static GtkWidgetClass *parent_class;
 
 #define DEFAULT_RADIUS 64
 #define DEFAULT_BORDER 2
+#define DEFAULT_HAND_WIDTH 3
 
+/* Color usage */
+#define SELECTED_BG_COLOR(widget)	 (& (widget)->style->base[GTK_STATE_SELECTED])
+#define SELECTED_FG_COLOR(widget)	 (& (widget)->style->text[GTK_STATE_SELECTED])
+
+/**
+ * GpeClockFace:
+ * 
+ * Object container for the GpeClockFace widget.
+ */
 struct _GpeClockFace
 {
   GtkWidget widget;
@@ -44,6 +55,9 @@ struct _GpeClockFace
   guint radius;
   guint border;
   double hand_width;
+  gboolean radius_set;
+  gboolean hand_width_set;
+  gboolean border_set;
 
   gboolean use_background_image;
   gboolean label_hours;
@@ -116,8 +130,7 @@ draw_hand (GpeClockFace *clock,
     }
 
 #ifdef HAVE_CAIRO
-  cairo_set_rgb_color (clock->cr, 0, 0, 0.8);
-  cairo_set_alpha (clock->cr, 0.5);
+  cairo_set_source_rgba (clock->cr, 0, 0, 0.8, 0.5);
 
   cairo_move_to (clock->cr, poly[0].x, poly[0].y);
   cairo_line_to (clock->cr, poly[1].x, poly[1].y);
@@ -195,42 +208,86 @@ gpe_clock_face_expose (GtkWidget *widget,
   gdk_gc_set_clip_rectangle (gc, NULL);
   gdk_gc_set_clip_rectangle (white_gc, NULL);
 
+#ifdef BACKGROUND_IMAGE
   if (clock->use_background_image)
     {
       GdkRectangle pixbuf_rect, intersect_rect;
       GdkPixbuf *current_background;
 
       if (afternoon)
-	current_background = clock_background_24;
+	{
+	  if (clock_background_24 == NULL)
+	    clock_background_24 = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/clock24.png", NULL);
+	  current_background = clock_background_24;
+	}
       else
-	current_background = clock_background;
+	{
+	  if (clock_background == NULL)
+	    clock_background = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/clock.png", NULL);
+	  current_background = clock_background;
+	}
+
+      pixbuf_rect.width = gdk_pixbuf_get_width (current_background);
+      pixbuf_rect.height = gdk_pixbuf_get_height (current_background);
+      pixbuf_rect.x = clock->x_offset + clock->radius - pixbuf_rect.width / 2;
+      pixbuf_rect.y = clock->y_offset + clock->radius - pixbuf_rect.width / 2;
       
       if (event)
 	{
-	  pixbuf_rect.x = clock->x_offset;
-	  pixbuf_rect.y = clock->y_offset;
-	  pixbuf_rect.width = background_width;
-	  pixbuf_rect.height = background_height;
-	  
 	  if (gdk_rectangle_intersect (&pixbuf_rect, &event->area, &intersect_rect) == TRUE)
+#if GTK_CHECK_VERSION(2,2,0)
+	    gdk_draw_pixbuf (clock->backing_pixmap,
+			     clock->backing_gc, 
+			     current_background, 
+			     intersect_rect.x - pixbuf_rect.x,
+			     intersect_rect.y - pixbuf_rect.y, 
+			     intersect_rect.x, intersect_rect.y,
+			     intersect_rect.width, intersect_rect.height,
+			     GDK_RGB_DITHER_NONE, 0, 0);
+#else
 	    gdk_pixbuf_render_to_drawable (current_background, 
 					   clock->backing_pixmap,
 					   clock->backing_gc, 
-					   intersect_rect.x - clock->x_offset, intersect_rect.y - clock->y_offset, 
+					   intersect_rect.x - pixbuf_rect.x, intersect_rect.y - pixbuf_rect.y, 
 					   intersect_rect.x, intersect_rect.y,
 					   intersect_rect.width, intersect_rect.height,
 					   GDK_RGB_DITHER_NONE, 0, 0);
+#endif
 	}
       else
-	gdk_pixbuf_render_to_drawable (current_background, 
-				       clock->backing_pixmap,
-				       clock->backing_gc, 
-				       0, 0, clock->x_offset, clock->y_offset,
-				       gdk_pixbuf_get_width (clock_background), 
-				       gdk_pixbuf_get_height (clock_background), 
-				       GDK_RGB_DITHER_NONE, 0, 0);
-      
+        {
+#if GTK_CHECK_VERSION(2,2,0)
+          gdk_draw_pixbuf (clock->backing_pixmap,
+                           clock->backing_gc,
+                           current_background,
+                           0, 0,
+			   pixbuf_rect.x,
+			   pixbuf_rect.y,
+			   -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+                         
+#else
+	  gdk_pixbuf_render_to_drawable (current_background, 
+				         clock->backing_pixmap,
+				         clock->backing_gc, 
+				         0, 0,
+			                 pixbuf_rect.x,
+                		         pixbuf_rect.y,
+				         -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+#endif
+        }
 
+      if (day_night_wheel == NULL)
+        day_night_wheel = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/day-night-wheel.png", NULL);
+
+#if GTK_CHECK_VERSION(2,2,0)
+      gdk_draw_pixbuf(clock->backing_pixmap, 
+		      clock->backing_gc, 
+		      day_night_wheel, 
+		      0, 0, 
+		      (clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2), 
+		      (clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2), 
+		      -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+#else
       gdk_pixbuf_render_to_drawable (day_night_wheel, 
 				     clock->backing_pixmap, 
 				     clock->backing_gc, 
@@ -238,7 +295,8 @@ gpe_clock_face_expose (GtkWidget *widget,
 				     (clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2), 
 				     (clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2), 
 				     -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
-      
+#endif
+
       gdk_draw_arc (clock->backing_pixmap, 
 		    widget->style->white_gc, TRUE,
 		    ((clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2)) - 2,
@@ -250,11 +308,11 @@ gpe_clock_face_expose (GtkWidget *widget,
 		    180 * 64);
     }
   else
+#endif /* BACKGROUND_IMAGE */
     {
 #ifdef HAVE_CAIRO
-      cairo_set_alpha (clock->cr, 1.0);
+      cairo_set_source_rgba (clock->cr, 1, 1, 1, 1.0);
 
-      cairo_set_rgb_color (clock->cr, 1, 1, 1);
       cairo_arc (clock->cr,
 		 clock->x_offset + clock->radius,
 		 clock->y_offset + clock->radius,
@@ -262,7 +320,8 @@ gpe_clock_face_expose (GtkWidget *widget,
 		 0,
 		 2 * M_PI);
       cairo_fill (clock->cr);
-      cairo_set_rgb_color (clock->cr, 0, 0, 0);
+      cairo_set_source_rgba (clock->cr, 0, 0, 0, 1.0);
+      cairo_set_line_width (clock->cr, clock->border);
       cairo_arc (clock->cr,
 		 clock->x_offset + clock->radius,
 		 clock->y_offset + clock->radius,
@@ -270,15 +329,25 @@ gpe_clock_face_expose (GtkWidget *widget,
 		 0,
 		 2 * M_PI);
       cairo_stroke (clock->cr);
+      cairo_arc (clock->cr,
+		 clock->x_offset + clock->radius,
+		 clock->y_offset + clock->radius,
+		 clock->hand_width,
+		 0,
+		 2 * M_PI);
+      cairo_fill (clock->cr);
 #else
+      gdk_gc_set_foreground (gc, SELECTED_BG_COLOR ((GtkWidget*)clock));
       gdk_draw_arc (clock->backing_pixmap, gc, TRUE,
 		    clock->x_offset, clock->y_offset,
 		    clock->radius * 2, clock->radius * 2,
 		    0, 360 * 64);
   
       gdk_draw_arc (clock->backing_pixmap, white_gc, TRUE,
-		    clock->x_offset + clock->border, clock->y_offset + clock->border,
-		    (clock->radius - clock->border) * 2, (clock->radius - clock->border) * 2,
+		    clock->x_offset + clock->border,
+		    clock->y_offset + clock->border,
+		    (clock->radius - clock->border) * 2,
+		    (clock->radius - clock->border) * 2,
 		    0, 360 * 64);
 #endif
 
@@ -297,19 +366,25 @@ gpe_clock_face_expose (GtkWidget *widget,
 	      
 	      angle = 2 * M_PI * i / 12;
 	      
-	      dx = sin (angle) * (clock->radius - clock->border - 10);
-	      dy = -cos (angle) * (clock->radius - clock->border - 10);
+	      dx = sin (angle) * (7 * clock->radius / 8 - clock->border);
+	      dy = -cos (angle) * (7 * clock->radius / 8 - clock->border);
 
 	      x = clock->x_offset + clock->radius + dx;
 	      y = clock->y_offset + clock->radius + dy;
 
-	      sprintf (buf, "%d", afternoon ? (i + 12) : i);
+	      sprintf (buf, "%d", afternoon ? (i ? i + 12 : 24) : (i ? i : 12));
 
 	      pango_layout_set_text (pl, buf, strlen (buf));
 	      pango_layout_get_size (pl, &width, &height);
 	      width /= PANGO_SCALE;
 	      height /= PANGO_SCALE;
 
+#if 1
+              gdk_gc_set_foreground (gc, SELECTED_FG_COLOR ((GtkWidget*)clock));
+              gdk_draw_layout(clock->backing_pixmap,
+                              gc,
+                              x - (width / 2), y - (height / 2), pl);
+#else
 	      gtk_paint_layout (widget->style,
 				clock->backing_pixmap,
 				GTK_WIDGET_STATE (widget),
@@ -318,6 +393,7 @@ gpe_clock_face_expose (GtkWidget *widget,
 				widget,
 				"label",
 				x - (width / 2), y - (height / 2), pl);
+#endif
 	    }
 	  
 	  g_object_unref (pl);
@@ -326,7 +402,7 @@ gpe_clock_face_expose (GtkWidget *widget,
 
   if (clock->second_adj != NULL)
     draw_hand (clock, clock->second_angle, 7 * clock->radius / 8, 1);
-  draw_hand (clock, clock->minute_angle, 7 * clock->radius / 8, clock->hand_width);
+  draw_hand (clock, clock->minute_angle, 9 * clock->radius / 10, clock->hand_width);
   draw_hand (clock, clock->hour_angle, 3 * clock->radius / 5, clock->hand_width);
 
   gdk_draw_drawable (drawable, gc, clock->backing_pixmap, 0, 0, 0, 0, 
@@ -488,11 +564,10 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
   gcm = gdk_drawable_get_colormap (widget->window);
       
 #ifdef HAVE_CAIRO
-  clock->cr = cairo_create ();
   clock->surface = cairo_xlib_surface_create (dpy, GDK_WINDOW_XWINDOW (clock->backing_pixmap), 
 					      gdk_x11_visual_get_xvisual (gv), 0, 
 					      gdk_x11_colormap_get_xcolormap (gcm));
-  cairo_set_target_surface (clock->cr, clock->surface);
+  clock->cr = cairo_create (clock->surface);
 #else
   clock->backing_poly_gc = gdk_gc_new (clock->backing_pixmap);
 #endif
@@ -527,10 +602,28 @@ gpe_clock_face_unprepare_xrender (GtkWidget *widget)
     }
 }
 
+static void
+gpe_clock_face_do_set_radius (GpeClockFace *clock, guint new_radius)
+{
+  clock->radius = new_radius;
+  gtk_widget_queue_resize (GTK_WIDGET (clock));
+	  
+  if (clock->radius != DEFAULT_RADIUS)
+    clock->use_background_image = FALSE;
+
+  if (!clock->hand_width_set)
+    clock->hand_width = new_radius / 40;
+
+  if (!clock->border_set)
+    clock->border = (new_radius < 16) ? DEFAULT_BORDER / 2 : DEFAULT_BORDER;
+}
+
 static void
 gpe_clock_face_size_allocate (GtkWidget     *widget,
 			      GtkAllocation *allocation)
 {
+  GpeClockFace *clock = GPE_CLOCK_FACE (widget);
+
   widget->allocation = *allocation;
 
   if (GTK_WIDGET_REALIZED (widget))
@@ -542,6 +635,18 @@ gpe_clock_face_size_allocate (GtkWidget     *widget,
       gpe_clock_face_unprepare_xrender (widget);
       gpe_clock_face_prepare_xrender (widget);
     }
+  
+  if (!clock->radius_set)
+    {
+      int min_size, new_radius;
+      
+      min_size = MIN (allocation->width, allocation->height);      
+
+      new_radius = (min_size / 2) - 1;
+
+      if (new_radius != clock->radius)
+	gpe_clock_face_do_set_radius (clock, new_radius);
+    }
 }
 
 static void
@@ -652,28 +757,17 @@ gpe_clock_face_size_request (GtkWidget	  *widget,
 static void
 gpe_clock_face_init (GpeClockFace *clock)
 {
-#ifdef BACKGROUND_IMAGE
-  if (clock_background == NULL)
-    clock_background = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/clock.png", NULL);
-  
-  if (clock_background_24 == NULL)
-    clock_background_24 = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/clock24.png", NULL);
-
-  background_width = gdk_pixbuf_get_width (clock_background);
-  background_height = gdk_pixbuf_get_height (clock_background);
-#endif
-  
-  if (day_night_wheel == NULL)
-    day_night_wheel = gdk_pixbuf_new_from_file (PREFIX "/share/libgpewidget/day-night-wheel.png", NULL);
-
-  // Double buffering doesn't play nicely with Render, so we will do that by hand
+  /* Double buffering doesn't play nicely with Render, so we will do that by hand */
   gtk_widget_set_double_buffered (GTK_WIDGET (clock), FALSE);
 
+  clock->radius_set = FALSE;
+  clock->hand_width_set = FALSE;
+  clock->border_set = FALSE;
   clock->radius = DEFAULT_RADIUS;
   clock->border = DEFAULT_BORDER;
+  clock->hand_width = DEFAULT_HAND_WIDTH;
   clock->use_background_image = TRUE;
   clock->label_hours = FALSE;
-  clock->hand_width = 3;
 
   clock->backing_pixmap = NULL;
   clock->backing_gc = NULL;
@@ -711,9 +805,15 @@ gpe_clock_face_set_do_grabs (GpeClockFace *clock, gboolean yes)
 void
 gpe_clock_face_set_radius (GpeClockFace *clock, guint radius)
 {
-  clock->radius = radius;
-  if (clock->radius != DEFAULT_RADIUS)
-    clock->use_background_image = FALSE;
+  clock->radius_set = TRUE;
+  gpe_clock_face_do_set_radius (clock, radius);
+}
+
+void
+gpe_clock_face_set_border (GpeClockFace *clock, guint border)
+{
+  clock->border_set = TRUE;
+  clock->border = border;
 }
 
 void
@@ -732,6 +832,7 @@ void
 gpe_clock_face_set_hand_width (GpeClockFace *clock, double width)
 {
   clock->hand_width = width;
+  clock->hand_width_set = TRUE;
 }
 
 GType
